#!/usr/bin/perl
#################################### Modules ##########################################
require 5.0.6;
use strict;
use warnings;
our $VERSION = v0.0.3;

################################### Variables #########################################
my @template_data;																											# array for storing the template
my @param_data;																													# array for storing the parameter data
my $file_count = 0;																											# for statistic

#######################################################################################
#################################### Main #############################################
#######################################################################################
get_options();
generate_config();
writeStatistic();
exit 0;

#######################################################################################
################################ Subroutines ##########################################
#######################################################################################
#get_options();
sub get_options{
my ($HELP) = <<EOFHELP;
Reads-in a Template- and parameterdata-File and generates multiple Config-Files
by replacing the placeholders <parameterx> with the corresponding value on the
parameter file. The config files will be named after the first parameter in the
data-file, so typically you should put the hostname as the first columns,
then files will be named hostnamex.cli

Program arguments:
	-h: just displays this help file
	-t: template file with placeholder like
		<parameterx>
	-p: parameter file with a comma or semicolon as separator, not a tab!
	    The first line must contain the names of the parameters
            e.g. parameterx (this name is then used as placeholder) 
Example:
	perl genconfig.pl -p data.csv -t template.cli
		
EOFHELP

	if(!defined($ARGV[0])){
		print $HELP;
		exit 1;
	}else{
		my $template;
		my $data;
		my $help;
		my $count = 0;
		foreach my $in (@ARGV){
			$template = $ARGV[$count + 1] if($in =~ /^-t$/);
			$data = $ARGV[$count + 1] if($in =~ /^-p$/);
			$help = 1 if($in =~ /^-h$/);
			$count++;
		}
		print $HELP if(defined($help));
		if((defined($template)) and (defined($data))){
			loadFile($template, \@template_data);
			loadFile($data, \@param_data);
		}else{
			print $HELP;
			exit 1;
		}
		
	}
}

#######################################################################################
#loadFile(<"datafile">);
sub loadFile{
	my $datafile = shift;
	my $data_out = shift;
	open DATA,"<$datafile" or die "Can not read File: \"$datafile\"!";
		@$data_out = <DATA>;
	close DATA;
}

#######################################################################################
#generate_config();
sub generate_config {
	my %value;
	my $header_value;
	my @header;
	
	chomp $param_data[0];																									# delete EOL unusable character 
	@header = split(/[,;]/,$param_data[0]);																# separate header line into array
	shift(@param_data);																										# delete header line
	
	foreach my $line (@param_data){																				# iterate through parameter file
		chomp $line;																												# delete EOL
		my @values = split(/[,;]/,$line);																		# create an array of parameter values
		my $filename = $values[0]."cli";																		# the name of the resulting config file
		my $count=0;
		foreach $header_value (@header){																		# iterate through the header line (i.e. parameter names)
				my $key = "<".$header_value.">";																# add brackets like in the template file
				$value{$key} = $values[$count];																	# generate a hash with the values 
				$count++;
		}

		$count = 0;
		my @current_output;
		my @current_data = @template_data;
		foreach my $templ_line (@current_data){
			if($templ_line =~ /<.*>/){																				# found the start & end of place holder
				while(my ($key, $param) = each %value){
					$templ_line =~ s/$key/$param/g if($templ_line =~ /$key/);			# replace the place holder global fashen
				}
			}
			$current_output[$count] = $templ_line;														# copy current line in to the output array
			$count++;
		}
		write_config ($filename,@current_output);
		$file_count++;			
	}
}

#######################################################################################
#writeStatistic();
sub writeStatistic{
	print "$file_count files generated.\n";
}
#######################################################################################
#write_config(<"config name">,<"config array">);
sub write_config {
	my ($config_name,@config) = @_;
	
	open(config_file,"> $config_name.cli");
	print config_file @config;
	close config_file;
}
